/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.yaml;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import org.junit.Test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;


import static org.junit.Assert.assertEquals;

public class JsonToYamlTest {

    private static final File API1_YAML_FILE = new File("./src/test/resources/systems-swagger.yaml");
    private static final File API1_JSON_FILE = new File("./src/test/resources/systems-swagger.json");

    @Test
    public void testConvert() throws IOException {

        // read source JSON file
        JsonObject goodjson = readFileToJson(API1_JSON_FILE).getAsJsonObject();

        final JsonToYaml j2y = new JsonToYaml();

        // convert to YAML string
        final String yamlStr = j2y.jsonToYamlString(goodjson);

        // get a temp file name
        final File tmpFile = mktempfile("JsonToYamlTest.testConvert", ".yaml");

        // write a new YAML file
        writeFile(tmpFile, yamlStr);

        // read the known good YAML file
        final YamlToJson y2j = new YamlToJson();
        final JsonObject goodyaml = y2j.yamlToJsonObject(API1_YAML_FILE);
        assertEquals(goodjson, goodyaml);

        // read the new YAML file
        final JsonObject testyaml = y2j.yamlToJsonObject(tmpFile);
        assertEquals(goodjson, testyaml);
    }

    private File mktempfile(final String prefix, final String suffix) throws IOException {
        final File f = File.createTempFile(prefix, suffix);
        f.deleteOnExit();
        return f;
    }

    private JsonElement readFileToJson(File f) throws IOException {
        try (final Reader in = new BufferedReader(new FileReader(f))) {
            final JsonParser parser = new JsonParser();
            return parser.parse(in);
        } catch (IOException e) {
            throw e;
        }
    }

    private void writeFile(File f, String contents) throws IOException {
        try (final Writer out = new BufferedWriter(new FileWriter(f))) {
            out.write(contents);
            out.close();
        } catch (IOException e) {
            throw e;
        }
    }
}
