/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.yaml;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.redhat.thermostat.common.json.JsonUtil;
import org.junit.Test;

import java.io.File;
import java.io.IOException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

public class RefResolverTest {

    private static final File API1_YAML_FILE = new File("./src/test/resources/systems-swagger.yaml");

    @Test
    public void resolverTest() throws IOException {
        YamlToJson y2j = new YamlToJson();
        JsonObject yaml = y2j.yamlToJsonObject(API1_YAML_FILE);
        assertNotNull(yaml);

        JsonElement getRefType = JsonUtil.fetch(yaml, "paths./systems/{systemId}.get.responses.200.schema.$ref");
        assertTrue(getRefType.getAsString().endsWith("definitions/systems-get-response"));

        // ref1 is the defintion pointed to by the reference
        JsonElement ref1 = JsonUtil.fetch(yaml, "definitions.systems-get-response");

        // resolve all refs
        RefResolver.resolveRefs(yaml, yaml);

        // get the response type, which should no longer be a reference but a copy of the definition pointed to by the reference
        JsonElement resolvedType = JsonUtil.fetch(yaml, "paths./systems/{systemId}.get.responses.200.schema");
        assertEquals(ref1, resolvedType);
    }
}
